/*
 * You own your strategy, and you have the right to modify, share or keep your strategy secret.
 * The license below is meant to protect Inovance from any liability and provide you the most 
 * freedom with your strategy.
 *
 * Copyright (c) 2015, Inovance, Inc.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this
 *  list of conditions and the following disclaimer.
 * 
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * * Neither the name of TRAIDE nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
//+------------------------------------------------------------------+
//|                                                   Inovance, Inc. |
//|                                     https://www.inovancetech.com |
//+------------------------------------------------------------------+

#property copyright ""
#property link      "https://www.inovancetech.com"
#property description "TRAIDE Exported Strategy"
#property version   "1.00"
#property strict

enum ExitCondition {
    BAR_CLOSE,    // Bar Close
    TP_SL         // TP/SL
};

// EA Inputs
input ExitCondition exitCondition = BAR_CLOSE;   // Exit Conditions
extern double  takeProfit         = 50.0;        // Take Profit (pips)
extern double  stopLoss           = 50.0;        // Stop Loss (pips)
input  double  lots               = 0.1;         // Lots
extern int     slippage           = 5;           // Maximum Slippage (pips)
input  int     magicNumber        = 1440174533;  // Magic Number

// Other global variables
double buyStopLossPips = 0.0;
double buyTakeProfitPips = 0.0;
double sellStopLossPips = 0.0;
double sellTakeProfitPips = 0.0;
datetime lastBar = D'1970.01.01 00:00';

int OnInit() {
    if (Bars < 30) {
        Print("Error initializing: Not enough bars in chart");
        return INIT_FAILED;
    }    
    if (exitCondition == TP_SL) {
        if (takeProfit < 10) {
            Print("Error initializing: Take Profit less than 10");
            return INIT_PARAMETERS_INCORRECT;
        }
        if(stopLoss < 10) {
            Print("Error initializing: Stop Loss less than 10");
            return INIT_PARAMETERS_INCORRECT;
        }
        if (Digits == 5 || Digits == 3) {
            takeProfit *= 10;
            stopLoss *= 10;
        }
    }    
    if (Digits == 5 || Digits == 3) {          
        slippage *= 10;
    }           
    Print("Exit Condition=", exitCondition, " Lots=", lots, " Take Profit=", takeProfit, " Stop Loss=", stopLoss, " Slippage=", slippage); 
    return INIT_SUCCEEDED;
}

void OnTick() {
    if (isNewBar()) {
        if (exitCondition == TP_SL) {
            setTakeProfitAndStopLoss();
        }
        int signal = traideRule0() + traideRule1();
        processSignal(signal);
    }
}

bool isNewBar() {
    if (lastBar != Time[0]) {
        lastBar = Time[0];
        return TRUE;
    }
    return FALSE;
} 

void setTakeProfitAndStopLoss() {
    buyTakeProfitPips = NormalizeDouble(Ask + takeProfit * Point, Digits);
    buyStopLossPips = NormalizeDouble(Ask - stopLoss * Point, Digits);
    sellTakeProfitPips = NormalizeDouble(Bid - takeProfit * Point, Digits);
    sellStopLossPips = NormalizeDouble(Bid + stopLoss * Point, Digits);
}

void processSignal(const int signal) {
    if (isPositionOpen()) {      
       processSignalHasPositionOpen(signal);
    } else {
       processSignalNoPositionOpen(signal);
    }
}

bool isPositionOpen() {
    for (int i = 0; i < OrdersTotal(); i++) {
        if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES) && OrderMagicNumber() == magicNumber && OrderSymbol() == Symbol()) { 
            return true;
       }
    }
    return false;
}

void processSignalHasPositionOpen(const int signal) {
    switch (OrderType()) {
    case OP_BUY:   // A buy order exists
        if (signal == 0) {
            Print("Long Conditions No Longer Met");
            if (!OrderClose(OrderTicket(), OrderLots(), Bid, slippage, Violet)) {
                Print("Error closing buy order: ", GetLastError());
            }
        } else if (signal < 0) {
            Print("Short Conditions Met");
            if (!OrderClose(OrderTicket(), OrderLots(), Bid, slippage, Violet)) {
            Print("Error closing buy order: ", GetLastError());
            } else if (OrderSend(Symbol(), OP_SELL, lots, Bid, slippage, sellStopLossPips, sellTakeProfitPips, "TRAIDE EA", magicNumber, 0, Red) < 0) {
                Print("Error opening sell order: ", GetLastError());
            }
        } else if (signal > 0) {
            Print("Long Conditions Still Met");
        }
        break;
    case OP_SELL:  // A sell order exists
        if (signal == 0) {
            Print("Short Conditions No Longer Met");
            if (!OrderClose(OrderTicket(), OrderLots(), Ask, slippage, Violet)) {
                Print("Error closing sell order: ", GetLastError());
            }
        } else if (signal > 0) {
            Print("Long Conditions Met");
            if (!OrderClose(OrderTicket(), OrderLots(), Ask, slippage, Violet)) {
                Print("Error closing sell order: ", GetLastError());
            } else if (OrderSend(Symbol(), OP_BUY, lots, Ask, slippage, buyStopLossPips, buyTakeProfitPips, "TRAIDE EA", magicNumber, 0, Green) < 0) {
                Print("Error opening buy order: ", GetLastError());
            }
        } else if (signal < 0) {
            Print("Short Conditions Still Met");
        }
        break;
    default:
         Print("Error, unexpected OrderType: ", OrderType());
         break;
    }
}

void processSignalNoPositionOpen(const int signal) {
    if (signal > 0) {
        Print("Long Conditions Met");
        if (OrderSend(Symbol(), OP_BUY, lots, Ask, slippage, buyStopLossPips, buyTakeProfitPips, "TRAIDE EA", magicNumber, 0, Green) < 0) {
            Print("Error opening buy order: ", GetLastError());
        }
    } else if (signal < 0) {
        Print("Short Conditions Met");
        if (OrderSend(Symbol(), OP_SELL, lots, Bid, slippage, sellStopLossPips, sellTakeProfitPips, "TRAIDE EA", magicNumber, 0, Red) < 0) {
            Print("Error opening sell order: ", GetLastError());
        }
    }
}

void OnDeinit(const int reason) {
    switch (reason) {
    case REASON_PROGRAM:
        Print("Closing EA: Expert Advisor terminated its operation by calling the ExpertRemove() function (REASON_PROGRAM).");
        closeEA();
        break;
    case REASON_REMOVE:
        Print("Closing EA: Program has been deleted from the chart (REASON_REMOVE).");
        closeEA();
        break;
    case REASON_RECOMPILE:
        Print("Reinitializing EA: Program has been recompiled (REASON_RECOMPILE).");
        reinitEA();
        break;
    case REASON_CHARTCHANGE:
        Print("Reinitializing EA: Symbol or chart period has been changed (REASON_CHARTCHANGE).");
        reinitEA();
        break;
    case REASON_CHARTCLOSE:
        Print("Closing: Chart has been closed (REASON_CHARTCLOSE).");
        closeEA();
        break;
    case REASON_PARAMETERS:
        Print("Reinitializing EA: Input parameters have been changed by user (REASON_PARAMETERS).");
        reinitEA();
        break;
    case REASON_ACCOUNT:
        Print("Reinitializing EA: Another account has been activated or reconnection to the trade server has occurred due to changes in the account settings (REASON_ACCOUNT).");
        reinitEA();
        break;
    case REASON_TEMPLATE:
         Print("Reinitializing EA: A new template has been applied (REASON_TEMPLATE).");
         reinitEA();
         break;
    case REASON_INITFAILED:
         Print("Closing EA: OnInit() handler has returned a nonzero value.");
         closeEA();
         break;
    case REASON_CLOSE:
         Print("Closing EA: Terminal has been closed.");
         closeEA();
         break;
    default:
         Print("Error, unexpected OnDeInit reason: ", reason);
         break;
    }
}

void closeEA() {
    if (isPositionOpen()) {
        switch (OrderType()) {
        case OP_BUY:   // A buy order exists
            if (!OrderClose(OrderTicket(), OrderLots(), Bid, slippage, Violet)) {
                Print("Error closing buy order: ", GetLastError());
            }
            break;
        case OP_SELL:  // A sell order exists
            if (!OrderClose(OrderTicket(), OrderLots(), Ask, slippage, Violet)) {
                Print("Error closing sell order: ", GetLastError());
            }
            break;
        default:
             Print("Error closeEA, unexpected OrderType: ", OrderType());
             break;
        }
    }    
}

void reinitEA() {
    reinitGlobals();
    if (OnInit() != INIT_SUCCEEDED) {
        Print("Closing EA: Reinitializing failed.");
        closeEA();
    }
}

void reinitGlobals() {
    buyStopLossPips = 0.0;
    buyTakeProfitPips = 0.0;
    sellStopLossPips = 0.0;
    sellTakeProfitPips = 0.0;
    lastBar = D'1970.01.01 00:00';
}

// Produced by TRAIDE Strategy Code Generator
int indCCI0period = 10;
int indMIN_MAX_RATIO1period = 10;
int indPPO2fastPeriod = 10;
int indPPO2slowPeriod = 25;
int indPPO2maType = 5;
int indROC3period = 5;
int indPRICE_MINUS_TRIMA4period = 10;
int traideRule0() {
    double CCI0 = NormalizeDouble(iCustom(NULL, PERIOD_D1, "TRAIDE-Indicators\\Technical\\Momentum\\CCI", indCCI0period, 0, 1), 5);
    double MIN_MAX_RATIO1 = NormalizeDouble(iCustom(NULL, PERIOD_D1, "TRAIDE-Indicators\\Technical\\Statistics\\MIN_MAX_RATIO", indMIN_MAX_RATIO1period, 0, 1), 5);
    double PPO2 = NormalizeDouble(iCustom(NULL, PERIOD_D1, "TRAIDE-Indicators\\Technical\\Momentum\\PPO", indPPO2fastPeriod, indPPO2slowPeriod, indPPO2maType, 0, 1), 5);
    double ROC3 = NormalizeDouble(iCustom(NULL, PERIOD_D1, "TRAIDE-Indicators\\Technical\\Momentum\\ROC", indROC3period, 0, 1), 5);
    double PRICE_MINUS_TRIMA4 = NormalizeDouble(iCustom(NULL, PERIOD_D1, "TRAIDE-Indicators\\Technical\\Overlap Studies\\Moving Averages\\Triangular Moving Average\\PRICE_MINUS_TRIMA", indPRICE_MINUS_TRIMA4period, 0, 1), 5);
    int match = 0;
    if (CCI0 > -197.86105849999998 && CCI0 <= -141.3293275) match++;
    else if (CCI0 > -141.3293275 && CCI0 <= -84.7975965) match++;
    else if (CCI0 > -84.7975965 && CCI0 <= -28.2658655) match++;
    if (MIN_MAX_RATIO1 > -4.956569 && MIN_MAX_RATIO1 <= 4.956569) match++;
    else if (MIN_MAX_RATIO1 > 4.956569 && MIN_MAX_RATIO1 <= 14.869707) match++;
    else if (MIN_MAX_RATIO1 > 14.869707 && MIN_MAX_RATIO1 <= 24.782845000000002) match++;
    if (PPO2 > -73.919184 && PPO2 <= -24.639728) match++;
    else if (PPO2 > -24.639728 && PPO2 <= 24.639728) match++;
    else if (PPO2 > 24.639728 && PPO2 <= 73.919184) match++;
    else if (PPO2 > 73.919184 && PPO2 <= 123.19864000000001) match++;
    if (ROC3 > -2.4845310000000005 && ROC3 <= -1.7746650000000006) match++;
    else if (ROC3 > -1.7746650000000002 && ROC3 <= -1.0647990000000003) match++;
    else if (ROC3 > -1.0647990000000003 && ROC3 <= -0.35493300000000005) match++;
    if (PRICE_MINUS_TRIMA4 > -80.98250099999998 && PRICE_MINUS_TRIMA4 <= -26.994166999999997) match++;
    if (match == 5) return 1;
    return 0;
}
int traideRule1() {
    double CCI0 = NormalizeDouble(iCustom(NULL, PERIOD_D1, "TRAIDE-Indicators\\Technical\\Momentum\\CCI", indCCI0period, 0, 1), 5);
    double MIN_MAX_RATIO1 = NormalizeDouble(iCustom(NULL, PERIOD_D1, "TRAIDE-Indicators\\Technical\\Statistics\\MIN_MAX_RATIO", indMIN_MAX_RATIO1period, 0, 1), 5);
    double PPO2 = NormalizeDouble(iCustom(NULL, PERIOD_D1, "TRAIDE-Indicators\\Technical\\Momentum\\PPO", indPPO2fastPeriod, indPPO2slowPeriod, indPPO2maType, 0, 1), 5);
    double ROC3 = NormalizeDouble(iCustom(NULL, PERIOD_D1, "TRAIDE-Indicators\\Technical\\Momentum\\ROC", indROC3period, 0, 1), 5);
    double PRICE_MINUS_TRIMA4 = NormalizeDouble(iCustom(NULL, PERIOD_D1, "TRAIDE-Indicators\\Technical\\Overlap Studies\\Moving Averages\\Triangular Moving Average\\PRICE_MINUS_TRIMA", indPRICE_MINUS_TRIMA4period, 0, 1), 5);
    int match = 0;
    if (CCI0 > 84.7975965 && CCI0 <= 141.3293275) match++;
    else if (CCI0 > 141.3293275 && CCI0 <= 197.86105849999998) match++;
    else if (CCI0 > 197.8610585 && CCI0 <= 254.3927895) match++;
    if (MIN_MAX_RATIO1 > 74.348535 && MIN_MAX_RATIO1 <= 84.261673) match++;
    else if (MIN_MAX_RATIO1 > 84.261673 && MIN_MAX_RATIO1 <= 94.174811) match++;
    else if (MIN_MAX_RATIO1 > 94.174811 && MIN_MAX_RATIO1 <= 104.08794900000001) match++;
    if (PPO2 > -24.639728 && PPO2 <= 24.639728) match++;
    else if (PPO2 > 24.639728 && PPO2 <= 73.919184) match++;
    if (ROC3 > 0.35493300000000005 && ROC3 <= 1.0647990000000003) match++;
    else if (ROC3 > 1.0647990000000003 && ROC3 <= 1.7746650000000002) match++;
    else if (ROC3 > 1.7746650000000006 && ROC3 <= 2.4845310000000005) match++;
    else if (ROC3 > 2.4845310000000005 && ROC3 <= 3.1943970000000004) match++;
    if (PRICE_MINUS_TRIMA4 > 26.994166999999997 && PRICE_MINUS_TRIMA4 <= 80.98250099999998) match++;
    else if (PRICE_MINUS_TRIMA4 > 80.98250099999998 && PRICE_MINUS_TRIMA4 <= 134.970835) match++;
    else if (PRICE_MINUS_TRIMA4 > 134.97083499999997 && PRICE_MINUS_TRIMA4 <= 188.95916899999997) match++;
    else if (PRICE_MINUS_TRIMA4 > 188.95916899999997 && PRICE_MINUS_TRIMA4 <= 242.94750299999998) match++;
    if (match == 5) return -1;
    return 0;
}
